#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
#from taskmanager import tm
import task_concat
def concat(vis=[''], concatvis='', freqtol='', dirtol='', respectname=False, timesort=False, copypointing=True, visweightscale=[]):

        """Concatenate several visibility data sets.

The list of data sets given in the vis argument are concatenated into an output
data set in concatvis.  If concatvis already exists (e.g., it is the same as the
first input data set), then the other input data sets will be appended to the
concatvis data set.  There is no limit to the number of input data sets.

If none of the input data sets have any scratch columns (model and corrected
columns), none are created in the concatvis.  Otherwise these columns are
created on output and initialized to their default value (1 in model column,
data in corrected column) for those data with no input columns.

Spectral windows for each data set with the same chanelization, and within a
specified frequency tolerance of another data set will be combined into one
spectral window.

A field position in one data set that is within a specified direction tolerance
of another field position in any other data set will be combined into one
field.  The field names need not be the same---only their position is used.

Each appended dataset is assigned a new observation id (provided the entries
in the observation table are indeed different).

Keyword arguments:
vis -- Name of input visibility files to be combined
	default: none; example: vis = ['src2.ms','ngc5921.ms','ngc315.ms']
concatvis -- Name of visibility file that will contain the concatenated data
	note: if this file exits on disk then the input files are 
              added to this file.  Otherwise the new file contains  
	      the concatenated data.  Be careful here when concatenating to
              an existing file.
	default: none; example: concatvis='src2.ms'
	         example: concatvis='outvis.ms'

        other examples: 
           concat(vis=['src2.ms','ngc5921.ms'], concatvis='src2.ms')
               will concatenate 'ngc5921.ms' into 'src2.ms', and the original
	       src2.ms is lost

	   concat(vis=['src2.ms','ngc5921.ms'], concatvis='out.ms') 
               will concatenate 'ngc5921.ms' and 'src2.ms' into a file named 
               'out.ms'; the original 'ngc5921.ms' and 'src2.ms' are untouched.

           concat(vis=['v1.ms','v2.ms'], concatvis = 'vall.ms')
              then
           concat(vis=['v3.ms','v4.ms'], concatvis = 'vall.ms')
             vall.ms will contains v1.ms+v2.ms+v3.ms+v4.ms

     Note: run flagmanager to save flags in the concatvis

freqtol -- Frequency shift tolerance for considering data to be in the same
           spwid.  The number of channels must also be the same.
	default: '' == 1 Hz
	example: freqtol='10MHz' will not combine spwid unless they are
	   within 10 MHz.
        Note: This option is useful to combine spectral windows with very slight
           frequency differences caused by Doppler tracking, for example.

dirtol -- Direction shift tolerance for considering data as the same field
	default: '' == 1 mas (milliarcsec)
	example: dirtol='1arcsec' will not combine data for a field unless
	   their phase center differ by less than 1 arcsec.  If the field names
           are different in the input data sets, the name in the output data
           set will be the first relevant data set in the list.

respectname -- If true, fields with a different name are not merged even if their 
        direction agrees (within dirtol)
        default: False

timesort -- If true, the output visibility table will be sorted in time.
        default: false.  Data in order as read in.
	example: timesort=true
     Note: There is no constraint on data that is simultaneously observed for
        more than one field; for example multi-source correlation of VLBA data.

copypointing -- Make a proper copy of the POINTING subtable (can be time consuming).
        If False, the result is an empty POINTING table.
        default: True

visweightscale -- The weights of the individual MSs will be scaled in the concatenated
        output MS by the factors in this list. Useful for handling heterogeneous arrays.
	Use plotms to inspect the "Wt" column as a reference for determining the scaling 
	factors. See the cookbook for more details.
	example: [1.,3.,3.] - scale the weights of the second and third MS by a factor 3.
	default: [] (empty list) - no scaling


        """
        if type(vis)==str: vis=[vis]
        if type(visweightscale)==float: visweightscale=[visweightscale]

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['concatvis'] = concatvis
        mytmp['freqtol'] = freqtol
        mytmp['dirtol'] = dirtol
        mytmp['respectname'] = respectname
        mytmp['timesort'] = timesort
        mytmp['copypointing'] = copypointing
        mytmp['visweightscale'] = visweightscale
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'concat.xml')

        casalog.origin('concat')
        if trec.has_key('concat') and casac.utils().verify(mytmp, trec['concat']) :
	    result = task_concat.concat(vis, concatvis, freqtol, dirtol, respectname, timesort, copypointing, visweightscale)

	else :
	  result = False
        return result
