#
# This file was generated using xslt from its XML file
#
# Copyright 2013, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from taskmanager import tm
from task_fluxscale import fluxscale
class fluxscale_cli_:
    __name__ = "fluxscale"
    __async__ = {}
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (fluxscale_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'vis':None, 'caltable':None, 'fluxtable':None, 'reference':None, 'transfer':None, 'listfile':None, 'append':None, 'refspwmap':None, 'incremental':None, 'fitorder':None,  'async':None}


    def result(self, key=None):
	    #### here we will scan the task-ids in __async__
	    #### and add any that have completed...
	    if key is not None and self.__async__.has_key(key) and self.__async__[key] is not None:
	       ret = tm.retrieve(self.__async__[key])
	       if ret['state'] == "done" :
	          self.__async__[key] = None
	       elif ret['state'] == 'crashed' :
		  self.__async__[key] = None
	       return ret
	    return None


    def __call__(self, vis=None, caltable=None, fluxtable=None, reference=None, transfer=None, listfile=None, append=None, refspwmap=None, incremental=None, fitorder=None,  async=None):

        """Bootstrap the flux density scale from standard calibrators

	Detailed Description: 


       Bootstrap the flux density scale from standard calibrators:

       After running gaincal on standard flux density calibrators (with or
       without an image model), and other calibrators with unknown flux
       densities (assumed 1 Jy), fluxscale applies the constraint that
       net system gain was, in fact, independent of field, on average,
       and that field-dependent gains in the input caltable are solely
       a result of the unknown flux densities for the calibrators.
       Using time-averaged gain amplitudes, the ratio between 
       each ordinary calibrator and the flux density calibrator(s) is 
       formed for each antenna and polarization (that they have in
       common).  The average of this ratio over antennas and polarizations
       yields a correction factor that is applied to the ordinary 
       calibrators' gains. (See also more detailed discussion in Example section below.)

	
	Arguments :
		vis:	Name of input visibility file (MS)
		   Default Value: 

		caltable:	Name of input calibration table
		   Default Value: 

		fluxtable:	Name of output, flux-scaled calibration table
		   Default Value: 

		reference:	Reference field name(s) (transfer flux scale FROM)
		   Default Value: 

		transfer:	Transfer field name(s) (transfer flux scale TO), \'\' -&gt; all
		   Default Value: 

		listfile:	Name of listfile that contains the fit information.  Default is '' (no file).
		   Default Value: 

		append:	Append solutions?
		   Default Value: False

		refspwmap:	Scale across spectral window boundaries.  See help fluxscale
		   Default Value: -1

		incremental:	incremental caltable
		   Default Value: False

		fitorder:	order of spectral fitting
		   Default Value: 1

	Returns: void

	Example :


       After running gaincal on standard flux density calibrators (with or
       without an image model), and other calibrators with unknown flux
       densities (assumed 1 Jy), fluxscale applies the constraint that
       net system gain was, in fact, independent of field, on average,
       and that field-dependent gains in the input caltable are solely
       a result of the unknown flux densities for the calibrators.
       Using time-averaged gain amplitudes, the ratio between 
       each ordinary calibrator and the flux density calibrator(s) is 
       formed for each antenna and polarization (that they have in
       common).  For incremental=False(default), the median of 
       this ratio over antennas and polarizations yields a correction 
       factor that is applied to the ordinary calibrators' gains. For 
       incremental=True, only the correction factors are written out 
       to the output fluxtable.

       The square of the gain correction factor for each calibrator
       and spw is the presumed flux density of that calibrator, and is
       reported in the logger.  The errors reported with this value
       reflect the scatter in gain ratio over antennas and
       polarizations, divided by the square root of the number of 
       antennas and polarizations available.  If the flux densities 
       for multiple spws exist, fitted spectral index and (for nspw>2)
       curvature are also reported. The fit is done for 
       log(flux density) = a_o + a_1*(log(frequency)) + a_2*(log(frequency))**2 
       where log(frequency) is with respect to the mean of log(frequency). 
       This reference frequency is reported in the logger along with the 
       flux density at that frequency.  The fit results are also reported 
       in the returned Python dictionary (the solutions are in 'spidx' 
       in the following order: a_o [log(S) at the zero point], a_1 [spectral index], 
       and a_2 [curvature]. And their errors are in 'spidxerr').  
       The MODEL_DATA column is currently _not_ revised to reflect the flux 
       densities derived by fluxscale.  Use setjy to set the MODEL_DATA 
       column, if necessary.

       The constant gain constraint is usually a reasonable assumption
       for the electronic systems on typical antennas.  It is
       important that external time- and/or elevation-dependent
       effects are separately accounted for when solving for the gain
       solution supplied to fluxscale, e.g., gain curves, 
       opacity, etc.  The fluxscale results can also be degraded
       by poor pointing during the observation.

       Keyword arguments:
       vis -- Name of input visibility file
               default: none; example: vis='ngc5921.ms'
       caltable -- Name of input calibration table
               default: none; example: caltable='ngc5921.gcal'
               This cal table was obtained from task gaincal.
       fluxtable -- Name of output, flux-scaled calibration table
               default: none; example: fluxtable='ngc5921.gcal2'
               The gains in this table have been adjusted by the
               derived flux density each calibrator.  The MODEL_DATA
               column has NOT been updated for the flux density of the
               calibrator.  Use setjy to do this if it is a point source.
       reference -- Reference field name(s)
               The names of the fields with a known flux densities or
                  visibilities that have been placed in the MODEL column
                  by setjy or ft for a model not in the CASA system.
               The syntax is similar to field.  Hence field index or
                  names can be used.
               default: none; example: reference='1328+307'
       transfer -- Transfer field name(s)
               The names of the fields with unknown flux densities.
                  These should be point-like calibrator sources
               The syntax is similar to field.  Hence source index or
                 names can be used.
               default: '' = all sources in caltable that are not specified
                  as reference sources.  Do not include unknown target sources
               example: transfer='1445+099, 3C84'; transfer = '0,4'

               NOTE: All fields in reference and transfer must have solutions
               in the caltable.

       listfile -- Fit listfile name
               The list file contains the flux density, flux density error,
                 S/N, and number of solutions (all antennas and feeds) for each
                 spectral window.  NOTE: The nominal spectral window frequencies
                 will be included in the future.
               default: '' = no fit listfile will be created.

       append -- Append fluxscaled solutions to the fluxtable.
               default: False; (will overwrite if already existing)
               example: append=True
       refspwmap -- Vector of spectral windows enabling scaling across
               spectral windows
               default: [-1]==> none.
               Example with 4 spectral windows:
               if the reference fields were observed only in spw=1 & 3,
               and the transfer fields were observed in all 4 spws (0,1,2,3),
               specify refspwmap=[1,1,3,3].
               This will ensure that transfer fields observed in spws 0,1,2,3
               will be referenced to reference field solutions only in
               spw 1 or 3.
      
       incremental -- Create an incremental caltable containing only gain correction 
               factors ( flux density= 1/(gain correction factor)**2)
               default: False; (older behavior = create flux-scaled gain table)
               example: incremental=True (output a caltable containing flux scale factors.)
              
               NOTE: If you use the incremental option, note that BOTH this incremental 
               fluxscale table AND an amplitude vs. time table should be supplied in applycal.

       fitorder -- Polynomial order of the spectral fitting for valid flux densities 
                with multiple spws.  Currently only support 1 (spectral index only) or 
                2 (spectral index and curvature).  It falls back to a lower fitorder if 
                there are not enough solutions to fit with the requested fitorder.
                default: 1

       Returned dictionary: 
                when it is run as fluxres = fluxscale(vis='my.ms',...), the determined flux
                densities and spectral index information  are returned as a Python dictionary with
                a format, {fieldIdstr: {spwIdstr: {'fluxd':array([I,Q,U,V]), 
                                                  'fluxdErr': corresponding errors,
                                                   'numSol': corresponding no. of solutions,
                                        'fieldName': field name,
                                        'fitFluxd': fitted flux density at the reference frequency,
                                        'fitFluxdErr': fitted flux density error,
                                        'fitRefFreq': reference frequency,
                                        'spidx': a_0, a_1, a_2
                                        'spidxerr': errors in a_0,a_1, a_2} 
                           'freq': (center) spw frequencies 
                           'spwID': list of spw IDs,
                           'spwName': list of spw names}, where fieldIdstr and spwIdstr
                are field Id and spw Id in string type, repectively.

       
 
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'fluxscale'
        self.__globals__['taskname'] = 'fluxscale'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')

	else :
            async = self.parameters['async']
            myparams['vis'] = vis = self.parameters['vis']
            myparams['caltable'] = caltable = self.parameters['caltable']
            myparams['fluxtable'] = fluxtable = self.parameters['fluxtable']
            myparams['reference'] = reference = self.parameters['reference']
            myparams['transfer'] = transfer = self.parameters['transfer']
            myparams['listfile'] = listfile = self.parameters['listfile']
            myparams['append'] = append = self.parameters['append']
            myparams['refspwmap'] = refspwmap = self.parameters['refspwmap']
            myparams['incremental'] = incremental = self.parameters['incremental']
            myparams['fitorder'] = fitorder = self.parameters['fitorder']

        if type(reference)==str: reference=[reference]
        if type(transfer)==str: transfer=[transfer]
        if type(refspwmap)==int: refspwmap=[refspwmap]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['caltable'] = caltable
        mytmp['fluxtable'] = fluxtable
        mytmp['reference'] = reference
        mytmp['transfer'] = transfer
        mytmp['listfile'] = listfile
        mytmp['append'] = append
        mytmp['refspwmap'] = refspwmap
        mytmp['incremental'] = incremental
        mytmp['fitorder'] = fitorder
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'fluxscale.xml')

        casalog.origin('fluxscale')
	try :
          #if not trec.has_key('fluxscale') or not casac.casac.utils().verify(mytmp, trec['fluxscale']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['fluxscale'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          saveinputs('fluxscale', 'fluxscale.last', myparams, self.__globals__,scriptstr=scriptstr)
          if async :
	    count = 0
	    keybase =  time.strftime("%y%m%d.%H%M%S")
	    key = keybase + "_" + str(count)
	    while self.__async__.has_key(key) :
	       count += 1
	       key = keybase + "_" + str(count)
            result = tm.execute('fluxscale', vis, caltable, fluxtable, reference, transfer, listfile, append, refspwmap, incremental, fitorder)
	    print "Use: "
	    print "      tm.retrieve(return_value) # to retrieve the status"
	    print 
	    self.rkey = key
	    self.__async__[key] = result
          else :
              tname = 'fluxscale'
              spaces = ' '*(18-len(tname))
              casalog.post('\n##########################################'+
                           '\n##### Begin Task: ' + tname + spaces + ' #####')
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
              result = fluxscale(vis, caltable, fluxtable, reference, transfer, listfile, append, refspwmap, incremental, fitorder)
              casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                           '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'fluxscale'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('fluxscale', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['vis']  = ''
        a['caltable']  = ''
        a['fluxtable']  = ''
        a['reference']  = ['']
        a['transfer']  = ['']
        a['listfile']  = ''
        a['append']  = False
        a['refspwmap']  = [-1]
        a['incremental']  = False
        a['fitorder']  = 1

        a['async']=False

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='fluxscale', subkey=None):
        desc={'fluxscale': 'Bootstrap the flux density scale from standard calibrators',
               'vis': 'Name of input visibility file (MS)',
               'caltable': 'Name of input calibration table',
               'fluxtable': 'Name of output, flux-scaled calibration table',
               'reference': 'Reference field name(s) (transfer flux scale FROM)',
               'transfer': 'Transfer field name(s) (transfer flux scale TO), \'\' -> all',
               'listfile': 'Name of listfile that contains the fit information.  Default is '' (no file).',
               'append': 'Append solutions?',
               'refspwmap': 'Scale across spectral window boundaries.  See help fluxscale',
               'incremental': 'incremental caltable',
               'fitorder': 'order of spectral fitting',

               'async': 'If true the taskname must be started using fluxscale(...)'
              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['caltable']  = ''
        a['fluxtable']  = ''
        a['reference']  = ['']
        a['transfer']  = ['']
        a['listfile']  = ''
        a['append']  = False
        a['refspwmap']  = [-1]
        a['incremental']  = False
        a['fitorder']  = 1

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if a.has_key(paramname) :
	      return a[paramname]
fluxscale_cli = fluxscale_cli_()
