#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
#from taskmanager import tm
import task_impv
def impv(imagename='', outfile='', start=[], end=[], width=1, unit='arcsec', overwrite=False, region="", chans='', stokes='', mask='', stretch=False):

        """Construct a position-velocity image by choosing two points in the direction plane.
PARAMETER SUMMARY
imagename        Name of the input (CASA, FITS, MIRIAD) image
outfile          Name of output CASA image. Must be specified.
start            The starting pixel in the direction plane (array of two values), such as [20, 5].
end              The ending pixel in the direction plane (array of two values), such as [200, 300].
width            Width, in pixels, for averaging pixels perpendicular to the slice.
                 This value is determined after rotating the image so the slice lies horizontally.
                 A value of one means no averaging. A value of 3 means average one pixel on each
                 side of the slice and the pixel on the slice. A value of 5 means average 2 pixels
                 on each side of the slice and the pixel on the slice, etc. Must be an odd integer >= 1.
unit             Allows the user to set the unit for the angular offset axis. Must be a unit of angular
                 measure.
overwrite        If output file is specified, this parameter controls if an already existing file by the
                 same name can be overwritten. If true, the user is not prompted, the file
                 if it exists is automatically overwritten.
region           Region specification. See help par.region. Default is to not use a region. If specified,
                 the entire direction plane must be specified. If specified do not specify chans or stokes.
chans            Optional contiguous frequency channel number specification. Default is all channels.
                 If specified, do not specify region. See "help par.chans" for examples. 
stokes           Contiguous stokes planes specification. If specified, do not specify region.
mask             Mask to use. See help par.mask. Default is none.
stretch          Stretch the input mask if necessary and possible. Only used if a mask is specified.
                 See help par.stretch.

Create a position-velocity image by specifying two pixels between which a slice is taken in the direction
coordinate. The spectral extent of the resulting image will be that provided by the region specification
or the entire spectral range of the input image if no region is specified. One may not specify a region
in direction space; that is accomplished by specifying the start and end points of the slice.
        
The start and end values represent direction pixel coordinates. They both must fail within the input
image, and they both must be at least 2 pixels from the edge of the input image to facilite rotation (see below).

One may specify a width (must be an odd integer >= 1), which is the number of pixels centered along and perpendicular
to the direction slice that are used for averaging along the slice. The default value of 1 represents no averaging).
A value of 3 means average one pixel on each side of the slice and the pixel on the slice. 
Note that this width is applied to pixels in the image after it has been rotated (see below for a description
of the algorithm used).
 
One may specify the unit for the angular offset axis.
        
Internally, the image is first rotated, padding if necessary to include relevant pixels that would otherwise
be excluded by the rotation operation, so that the slice is horizontal, with the starting pixel left of the
ending pixel. Then, the pixels within the specified width of the slice are averaged and the resulting image is
written and/or returned. The output image has a linear coordinate in place of the direction coordinate of the
input image, and the corresponding axis represents angular offset with the center pixel having a value of 0.
        
The equivalent coordinate system, with a (usually) rotated direction coordinate (eg, RA and Dec) is written
to the output image as a table record. It can be retrieved using the table tool as shown in the example below.
        
# create a pv image with the position axis running from ra, dec pixel positions of [45, 50] to [100, 120]
# in the input image
impv(imagename="my_spectral_cube.im", outfile="mypv.im", start=[45,50], end=[100,120])
# analyze the pv image, such as get statistics
pvstats = imstat("mypv.im")
#
# get the alternate coordinate system information
tb.open("mypv.im")
alternate_csys_record = tb.getkeyword("misc")["secondary_coordinates"]
tb.done()
    
        """
        if type(start)==float: start=[start]
        if type(end)==float: end=[end]

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['outfile'] = outfile
        mytmp['start'] = start
        mytmp['end'] = end
        mytmp['width'] = width
        mytmp['unit'] = unit
        mytmp['overwrite'] = overwrite
        mytmp['region'] = region
        mytmp['chans'] = chans
        mytmp['stokes'] = stokes
        mytmp['mask'] = mask
        mytmp['stretch'] = stretch
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'impv.xml')

        casalog.origin('impv')
        if trec.has_key('impv') and casac.utils().verify(mytmp, trec['impv']) :
	    result = task_impv.impv(imagename, outfile, start, end, width, unit, overwrite, region, chans, stokes, mask, stretch)

	else :
	  result = False
        return result
