#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_impv import impv
from task_impv import casalog

class impv_pg_:
    __name__ = "impv"

    def __init__(self) :
       self.__bases__ = (impv_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, imagename=None, outfile=None, start=None, end=None, width=None, unit=None, overwrite=None, region=None, chans=None, stokes=None, mask=None, stretch=None, async=None):

        """Construct a position-velocity image by choosing two points in the direction plane.
PARAMETER SUMMARY
imagename        Name of the input (CASA, FITS, MIRIAD) image
outfile          Name of output CASA image. Must be specified.
start            The starting pixel in the direction plane (array of two values), such as [20, 5].
end              The ending pixel in the direction plane (array of two values), such as [200, 300].
width            Width, in pixels, for averaging pixels perpendicular to the slice.
                 This value is determined after rotating the image so the slice lies horizontally.
                 A value of one means no averaging. A value of 3 means average one pixel on each
                 side of the slice and the pixel on the slice. A value of 5 means average 2 pixels
                 on each side of the slice and the pixel on the slice, etc. Must be an odd integer >= 1.
unit             Allows the user to set the unit for the angular offset axis. Must be a unit of angular
                 measure.
overwrite        If output file is specified, this parameter controls if an already existing file by the
                 same name can be overwritten. If true, the user is not prompted, the file
                 if it exists is automatically overwritten.
region           Region specification. See help par.region. Default is to not use a region. If specified,
                 the entire direction plane must be specified. If specified do not specify chans or stokes.
chans            Optional contiguous frequency channel number specification. Default is all channels.
                 If specified, do not specify region. See "help par.chans" for examples. 
stokes           Contiguous stokes planes specification. If specified, do not specify region.
mask             Mask to use. See help par.mask. Default is none.
stretch          Stretch the input mask if necessary and possible. Only used if a mask is specified.
                 See help par.stretch.

Create a position-velocity image by specifying two pixels between which a slice is taken in the direction
coordinate. The spectral extent of the resulting image will be that provided by the region specification
or the entire spectral range of the input image if no region is specified. One may not specify a region
in direction space; that is accomplished by specifying the start and end points of the slice.
        
The start and end values represent direction pixel coordinates. They both must fail within the input
image, and they both must be at least 2 pixels from the edge of the input image to facilite rotation (see below).

One may specify a width (must be an odd integer >= 1), which is the number of pixels centered along and perpendicular
to the direction slice that are used for averaging along the slice. The default value of 1 represents no averaging).
A value of 3 means average one pixel on each side of the slice and the pixel on the slice. 
Note that this width is applied to pixels in the image after it has been rotated (see below for a description
of the algorithm used).
 
One may specify the unit for the angular offset axis.
        
Internally, the image is first rotated, padding if necessary to include relevant pixels that would otherwise
be excluded by the rotation operation, so that the slice is horizontal, with the starting pixel left of the
ending pixel. Then, the pixels within the specified width of the slice are averaged and the resulting image is
written and/or returned. The output image has a linear coordinate in place of the direction coordinate of the
input image, and the corresponding axis represents angular offset with the center pixel having a value of 0.
        
The equivalent coordinate system, with a (usually) rotated direction coordinate (eg, RA and Dec) is written
to the output image as a table record. It can be retrieved using the table tool as shown in the example below.
        
# create a pv image with the position axis running from ra, dec pixel positions of [45, 50] to [100, 120]
# in the input image
impv(imagename="my_spectral_cube.im", outfile="mypv.im", start=[45,50], end=[100,120])
# analyze the pv image, such as get statistics
pvstats = imstat("mypv.im")
#
# get the alternate coordinate system information
tb.open("mypv.im")
alternate_csys_record = tb.getkeyword("misc")["secondary_coordinates"]
tb.done()
    
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'impv'
        myf['taskname'] = 'impv'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['imagename'] = imagename = myf['imagename']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['start'] = start = myf['start']
            myparams['end'] = end = myf['end']
            myparams['width'] = width = myf['width']
            myparams['unit'] = unit = myf['unit']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['region'] = region = myf['region']
            myparams['chans'] = chans = myf['chans']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['mask'] = mask = myf['mask']
            myparams['stretch'] = stretch = myf['stretch']

        if type(start)==float: start=[start]
        if type(end)==float: end=[end]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['outfile'] = outfile
        mytmp['start'] = start
        mytmp['end'] = end
        mytmp['width'] = width
        mytmp['unit'] = unit
        mytmp['overwrite'] = overwrite
        mytmp['region'] = region
        mytmp['chans'] = chans
        mytmp['stokes'] = stokes
        mytmp['mask'] = mask
        mytmp['stretch'] = stretch
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'impv.xml')

        casalog.origin('impv')
        if not trec.has_key('impv') or not casac.utils().verify(mytmp, trec['impv']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: impv           #####')
          casalog.post('')
          result = impv(imagename, outfile, start, end, width, unit, overwrite, region, chans, stokes, mask, stretch)
          casalog.post('')
          casalog.post('##### End Task: impv           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('impv', 'impv.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'impv'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('impv', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['imagename']  = ''
        a['outfile']  = ''
        a['start']  = []
        a['end']  = []
        a['width']  = 1
        a['unit']  = 'arcsec'
        a['chans']  = ''
        a['stokes']  = ''
        a['mask']  = ''

        a['async']=False
        a['outfile'] = {
                    0:odict([{'notvalue':''}, {'overwrite':False}])}
        a['mask'] = {
                    0:odict([{'notvalue':''}, {'stretch':False}])}
        a['chans'] = {
                    0:odict([{'value':''}, {'region':""}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='impv', subkey=None):
        desc={'impv': 'Construct a position-velocity image by choosing two points in the direction plane.',
               'imagename': 'Name of the input image',
               'outfile': 'Output image name. If empty, no image is written.',
               'start': 'The starting pixel in the direction plane (array of two values).',
               'end': 'The ending pixel in the direction plane (array of two values).',
               'width': 'Width in pixels for averaging pixels perpendicular to the slice. Must be an odd integer >= 1 (1 means only use the pixels along the slice).',
               'unit': 'Unit for the offset axis in the resulting image. Must be a unit of angular measure.',
               'overwrite': 'Overwrite the output if it exists?',
               'region': 'Region selection. Default is entire image. No selection is permitted in the direction plane. See help par.region.',
               'chans': 'Channels to use. See "help par.chans" for examples. Channels must be contiguous. Default is to use all channels.',
               'stokes': 'Stokes planes to use. Planes must be contiguous. Default is to use all stokes.',
               'mask': 'Mask to use. See help par.mask. Default is none.',
               'stretch': 'Stretch the mask if necessary and possible? See help par.stretch. Default False',

               'async': 'If true the taskname must be started using impv(...)'
              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['outfile']  = ''
        a['start']  = []
        a['end']  = []
        a['width']  = 1
        a['unit']  = 'arcsec'
        a['overwrite']  = False
        a['region']  = ""
        a['chans']  = ''
        a['stokes']  = ''
        a['mask']  = ''
        a['stretch']  = False

        if a.has_key(paramname) :
	      return a[paramname]
impv_pg = impv_pg_()
