#
# This file was generated using xslt from its XML file
#
# Copyright 2013, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from taskmanager import tm
from task_imsmooth import imsmooth
class imsmooth_cli_:
    __name__ = "imsmooth"
    __async__ = {}
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (imsmooth_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'imagename':None, 'kernel':None, 'major':None, 'minor':None, 'pa':None, 'targetres':None, 'region':None, 'box':None, 'chans':None, 'stokes':None, 'mask':None, 'outfile':None, 'stretch':None, 'overwrite':None, 'beam':None,  'async':None}


    def result(self, key=None):
	    #### here we will scan the task-ids in __async__
	    #### and add any that have completed...
	    if key is not None and self.__async__.has_key(key) and self.__async__[key] is not None:
	       ret = tm.retrieve(self.__async__[key])
	       if ret['state'] == "done" :
	          self.__async__[key] = None
	       elif ret['state'] == 'crashed' :
		  self.__async__[key] = None
	       return ret
	    return None


    def __call__(self, imagename=None, kernel=None, major=None, minor=None, pa=None, targetres=None, region=None, box=None, chans=None, stokes=None, mask=None, outfile=None, stretch=None, overwrite=None, beam=None,  async=None):

        """Smooth an image or portion of an image

	Detailed Description: 


	Arguments :
		imagename:	Name of the input image. Must be specified.
		   Default Value: 

		kernel:	Type of kernel to use: gaussian or boxcar.
		   Default Value: gauss

		major:	Major axis for the kernels. Standard quantity representation. Must be specified for kernel="boxcar". Example: "4arcsec".
		   Default Value: 

		minor:	Minor axis. Standard quantity representation. Must be specified for kernel="boxcar". Example: "2arcsec".
		   Default Value: 

		pa:	Position angle used only for gaussian kernel. Standard quantity representation. Example: "40deg".
		   Default Value: 

		targetres:	If gaussian kernel, specified parameters are to be resolution of output image (True) or parameters of gaussian to convolve with input image (False).
		   Default Value: False

		region:	Region selection. See help par.region. Empty string means use box/chans/stokes if supplied, or else entire image.
		   Default Value: 

		box:	Rectangular region specification in directional plane. Do not specify region if you specify box.
		   Default Value: 

		chans:	Select the spectral channel range. See "help par.chans" for examples. Do not specify region if you specify chans.
		   Default Value: 

		stokes:	Stokes parameters to image (eg, I,IV,IQU,IQUV). Do not specify region if you specify stokes.
		   Default Value: 

		mask:	Mask to use. See help par.mask. Default is none.
		   Default Value: 

		outfile:	Output image name. Must be specified.
		   Default Value: 

		stretch:	Stretch the mask if necessary and possible? See help par.stretch 
		   Default Value: False

		overwrite:	Overwrite (unprompted) pre-existing output file?
		   Default Value: False

		beam:	Alternate way of describing a Gaussian. If specified, must be a dictionary with keys "major", "minor", and "pa" (or "positionangle"). Do not specify beam if specifying major, minor, and pa. Example: Example: {"major": "5arcsec", "minor": "2arcsec", "pa": "20deg"}.
		   Default Value: 

	Returns: any

	Example :

This task performs a Fourier-based convolution to 'smooth' the
image. Smoothing is typically performed in order to reduce the noise in
an image.

Keyword arguments:
	
imagename    Input image name. Must be specified.
outfile      Output smoothed image file name. Must be specified.
kernel       Type of kernel to use when smoothing (gaussian or boxcar).
             For boxcar smoothing, the major axis is parallel to the y-axis of the image
             and the minor axis is parallel to the x-axis. For a Gaussian, the
             orientation is specified by a position angle. A value of 0 degrees means
             the major axis is parallel to the y-axis and an increasing value of the
             position angle results in a counter-clockwise rotation of the ellipse.
                default: 'gauss'; example: kernel='gauss'
                options: 'gauss' or 'boxcar' 
major        Major axis of kernel which must be specified for boxcar smoothing. For
             Gaussian smoothing, the kernel parameters can alternatively be specified
             in the beam parameter. Standard quantity representations are supported.
             Example "4arcsec".
minor        Minor axis of kernel which must be specified for boxcar smoothing. For
             Gaussian smoothing, the kernel parameters can alternatively be specified
             in the beam parameter. Standard quantity representations are supported.
             Example "3arcsec".
pa           Position angle to use for gaussian kernel, unused for boxcar. 
             The Gaussian kernel parameters can alternatively be specified
             in the beam parameter. Standard quantity representations are supported.
             Example "40deg".
beam         Record specifying Gaussian beam parameters. Do not specify any of
             major, minor, or pa if you choose to specify this parameter.
             Example: {"major": "5arcsec", "minor": "2arcsec", "pa": "20deg"}
targetres    Boolean used only for gaussian kernels. If True, kernel parameters (major/minor/pa
             or beam) are the resolution of the output image. If false, a gaussian
             with these parameters is convolved with the input image to produce
             the output image.
mask         Mask to use. See help par.mask. Default is none.
region       Region selection in the input image. See help par.region for details.
             You may specify none or one of region or a box/chans/stokes combination, but not both.
            
box          A rectangular region on the directional plane. Four comma seperated non-negative numbers,
             the first two representing the blc and the last two representing the trc, in pixels.
             The specified corners must be located within the image. Empty string means use the
             full directional plane.
             Example: "5, 10, 100, 200".
             
chans        Channel selection. See help par.chans for details.
             See "help par.chans" for examples. Empty string means use all channels.
stokes       Stokes selection. Empty string means use all stokes.
             Example: 'I'
             Options: 'I','Q','U','V','RR','RL','LR','LL','XX','YX','XY','YY', ... 



GAUSSIAN KERNEL

Major and minor are the full width at half maximum  (FWHM) of the Gaussian. pa is the position angle
of the Gaussian. The beam parameter offers an alternate way of describing the convolving Gaussian.
If used, neither major, minor, nor pa can be specified. The beam parameter must have exactly three
fields: "major", "minor", and "pa" (or "positionangle"). This is the record format for the output
of ia.restoringbeam(). For example

beam = {"major": "5arcsec", "minor": "2arcsec", "pa": "20deg"}

If both beam and any of major, minor, and/or pa is specified for a Gaussian kernel,
an exception will be thrown.    

In addition, the targetres parameter indicates if the specified Gaussian is to be the
resolution of the final image (True) or if it is to be used to convolve the input image.
If True, the input image must have a restoring beam. Use imhead() or ia.restoringbeam()
to check for its existence. If the image has multiple beams and targetres=True,
all planes in the image will be convolved so that the resulting resolution is that
specified by the kernel parameters. If the image has multiple beams and targetres=False,
each plane will be convolved with a Gaussian specified by beam (and hence, in
general, the output image will also have multiple beams that vary with spectral channel
and/or polarization).

BOXCAR KERNEL

major is length of the box along the y-axis and minor is length of the box along the x-axis.
pa is not used and beam should not be specified. The value of targetres is not used.

IN GENERAL

The major, minor, and pa parameters can be specified in one of three ways
   Quantity -- for example major=qa.quantity(1, 'arcsec')
               Note that you can use pixel units, such as 
               major=qa.quantity(1, 'pix')
   String -- for example minor='1pix' or major='0.5arcsec'
             (i.e. a string that the Quanta quantity function accepts).
   Numeric -- for example major=10.
              In this case, the units of major and minor are assumed to 
              be in arcsec and units of pa are assumed to be degrees. 

Note: Using pixel units allows you to convolve axes with different units.

EXAMPLES

# smoothing with a gaussian kernel 20arseconds by 10 arseconds
imsmooth( imagename='my.image', kernel='gauss', major='20arcsec', minor='10arcsec', pa="0deg")

# the same as before, just a different way of specifying the kernel parameters
mybeam = {'major': '20arcsec', 'minor': '10arcsec', 'pa': '0deg'}
imsmooth( imagename='my.image', kernel='gauss', beam=mybeam)

# Smoothing using pixel coordinates and a boxcar kernel.
imsmooth( imagename='new.image', major='20pix', minor='10pix', kernel='boxcar')


        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'imsmooth'
        self.__globals__['taskname'] = 'imsmooth'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')

	else :
            async = self.parameters['async']
            myparams['imagename'] = imagename = self.parameters['imagename']
            myparams['kernel'] = kernel = self.parameters['kernel']
            myparams['major'] = major = self.parameters['major']
            myparams['minor'] = minor = self.parameters['minor']
            myparams['pa'] = pa = self.parameters['pa']
            myparams['targetres'] = targetres = self.parameters['targetres']
            myparams['region'] = region = self.parameters['region']
            myparams['box'] = box = self.parameters['box']
            myparams['chans'] = chans = self.parameters['chans']
            myparams['stokes'] = stokes = self.parameters['stokes']
            myparams['mask'] = mask = self.parameters['mask']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['stretch'] = stretch = self.parameters['stretch']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']
            myparams['beam'] = beam = self.parameters['beam']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['kernel'] = kernel
        mytmp['major'] = major
        mytmp['minor'] = minor
        mytmp['pa'] = pa
        mytmp['targetres'] = targetres
        mytmp['region'] = region
        mytmp['box'] = box
        mytmp['chans'] = chans
        mytmp['stokes'] = stokes
        mytmp['mask'] = mask
        mytmp['outfile'] = outfile
        mytmp['stretch'] = stretch
        mytmp['overwrite'] = overwrite
        mytmp['beam'] = beam
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'imsmooth.xml')

        casalog.origin('imsmooth')
	try :
          #if not trec.has_key('imsmooth') or not casac.casac.utils().verify(mytmp, trec['imsmooth']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['imsmooth'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          saveinputs('imsmooth', 'imsmooth.last', myparams, self.__globals__,scriptstr=scriptstr)
          if async :
	    count = 0
	    keybase =  time.strftime("%y%m%d.%H%M%S")
	    key = keybase + "_" + str(count)
	    while self.__async__.has_key(key) :
	       count += 1
	       key = keybase + "_" + str(count)
            result = tm.execute('imsmooth', imagename, kernel, major, minor, pa, targetres, region, box, chans, stokes, mask, outfile, stretch, overwrite, beam)
	    print "Use: "
	    print "      tm.retrieve(return_value) # to retrieve the status"
	    print 
	    self.rkey = key
	    self.__async__[key] = result
          else :
              tname = 'imsmooth'
              spaces = ' '*(18-len(tname))
              casalog.post('\n##########################################'+
                           '\n##### Begin Task: ' + tname + spaces + ' #####')
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
              result = imsmooth(imagename, kernel, major, minor, pa, targetres, region, box, chans, stokes, mask, outfile, stretch, overwrite, beam)
              casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                           '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'imsmooth'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('imsmooth', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['imagename']  = ''
        a['kernel']  = 'gauss'
        a['major']  = ''
        a['minor']  = ''
        a['pa']  = ''
        a['region']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''
        a['mask']  = ''
        a['outfile']  = ''
        a['overwrite']  = False

        a['async']=False
        a['kernel'] = {
                    0:odict([{'value':'gauss'}, {'beam':''}, {'targetres':False}]), 
                    1:odict([{'value':'gaussian'}, {'beam':''}, {'targetres':False}])}
        a['mask'] = {
                    0:odict([{'notvalue':''}, {'stretch':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='imsmooth', subkey=None):
        desc={'imsmooth': 'Smooth an image or portion of an image',
               'imagename': 'Name of the input image. Must be specified.',
               'kernel': 'Type of kernel to use: gaussian or boxcar.',
               'major': 'Major axis for the kernels. Standard quantity representation. Must be specified for kernel="boxcar". Example: "4arcsec".',
               'minor': 'Minor axis. Standard quantity representation. Must be specified for kernel="boxcar". Example: "2arcsec".',
               'pa': 'Position angle used only for gaussian kernel. Standard quantity representation. Example: "40deg".',
               'targetres': 'If gaussian kernel, specified parameters are to be resolution of output image (True) or parameters of gaussian to convolve with input image (False).',
               'region': 'Region selection. See help par.region. Empty string means use box/chans/stokes if supplied, or else entire image.',
               'box': 'Rectangular region specification in directional plane. Do not specify region if you specify box.',
               'chans': 'Select the spectral channel range. See "help par.chans" for examples. Do not specify region if you specify chans.',
               'stokes': 'Stokes parameters to image (eg, I,IV,IQU,IQUV). Do not specify region if you specify stokes.',
               'mask': 'Mask to use. See help par.mask. Default is none.',
               'outfile': 'Output image name. Must be specified.',
               'stretch': 'Stretch the mask if necessary and possible? See help par.stretch ',
               'overwrite': 'Overwrite (unprompted) pre-existing output file?',
               'beam': 'Alternate way of describing a Gaussian. If specified, must be a dictionary with keys "major", "minor", and "pa" (or "positionangle"). Do not specify beam if specifying major, minor, and pa. Example: Example: {"major": "5arcsec", "minor": "2arcsec", "pa": "20deg"}.',

               'async': 'If true the taskname must be started using imsmooth(...)'
              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['kernel']  = 'gauss'
        a['major']  = ''
        a['minor']  = ''
        a['pa']  = ''
        a['targetres']  = False
        a['region']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''
        a['mask']  = ''
        a['outfile']  = ''
        a['stretch']  = False
        a['overwrite']  = False
        a['beam']  = ''

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['kernel']  == 'gauss':
            a['beam'] = ''
            a['targetres'] = False

        if self.parameters['kernel']  == 'gaussian':
            a['beam'] = ''
            a['targetres'] = False

        if self.parameters['mask']  != '':
            a['stretch'] = False

        if a.has_key(paramname) :
	      return a[paramname]
imsmooth_cli = imsmooth_cli_()
