#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_mstransform import mstransform
from task_mstransform import casalog

class mstransform_pg_:
    __name__ = "mstransform"

    def __init__(self) :
       self.__bases__ = (mstransform_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None, outputvis=None, createmms=None, separationaxis=None, numsubms=None, parallel=None, ddistart=None, tileshape=None, field=None, spw=None, scan=None, antenna=None, correlation=None, timerange=None, intent=None, array=None, uvrange=None, observation=None, feed=None, datacolumn=None, realmodelcol=None, usewtspectrum=None, combinespws=None, chanaverage=None, chanbin=None, useweights=None, hanning=None, regridms=None, mode=None, nchan=None, start=None, width=None, nspw=None, interpolation=None, phasecenter=None, restfreq=None, outframe=None, veltype=None, timeaverage=None, timebin=None, timespan=None, maxuvwdistance=None, async=None):

        """Split the MS, combine/separate/regrid spws and do channel and time averaging  

    Detailed description of keyword arguments:

--- Input/Output parameters ---
    vis -- Name of input visibility file
        default: ''; example: vis='ngc5921.ms'

    outputvis -- Name of output visibility file
        default: ''; example: outputvis='ngc5921.mms'

    createmms -- Create a multi-MS as the output.
        default: False

        If False, it will work like the split task and create a
        normal MS, split according to the given data selection parameters.
        Note that, when this parameter is set to False, a simple_cluster
        will not be created.

        separationaxis -- Axis to do parallelization across. 
            default: 'both'
            options: 'scan', 'spw', 'both'
            The 'both' option will try to partition in both scan and spw axes.

        numsubms -- The number of sub-MSs to create (int).
            default: 64

        parallel -- Create a multi-MS in parallel, using the simple_cluster
                    infrastructure. It will run the tool several times in parallel,
                    one instance for each subMS. Note that for small MSs, we
                    recommend to not change this parameter as the overhead of creating
                    a cluster may be larger than the gain to run in parallel. (bool)
            default: False
                
    tileshape -- List with 1 or 3 elements describing the tile shape that will be used
                 to save the columns to disk. (list)
        default: [0]
        options: [0] or [1] or [int,int,int]. When list has only one element, it should
                 be either 0 or 1. When the list has three elements, they should be the
                 number of correlations, channels, rows.


--- Data selection parameters ---
    field -- Select field using field id(s) or field name(s).
             [run listobs to obtain the list iof d's or names]
        default: ''=all fields If field string is a non-negative
           integer, it is assumed to be a field index
           otherwise, it is assumed to be a field name
           field='0~2'; field ids 0,1,2
           field='0,4,5~7'; field ids 0,4,5,6,7
           field='3C286,3C295'; fields named 3C286 and 3C295
           field = '3,4C*'; field id 3, all names starting with 4C

    spw -- Select spectral window/channels
        default: ''=all spectral windows and channels
           spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
           spw='<2';  spectral windows less than 2 (i.e. 0,1)
           spw='0:5~61'; spw 0, channels 5 to 61
           spw='0,10,3:3~45'; spw 0,10 all channels, spw 3 - chans 3 to 45.
           spw='0~2:2~6'; spw 0,1,2 with channels 2 through 6 in each.
           spw = '*:3~64'  channels 3 through 64 for all sp id's
                   spw = ' :3~64' will NOT work.

               NOTE: mstransform does not support multiple channel ranges per
                     spectral window (';').

    scan -- Scan number range
        default: ''=all

    antenna -- Select data based on antenna/baseline
        default: '' (all)
            Non-negative integers are assumed to be antenna indices, and
            anything else is taken as an antenna name.

        examples:
            antenna='5&6': baseline between antenna index 5 and index 6.
            antenna='VA05&VA06': baseline between VLA antenna 5 and 6.
            antenna='5&6;7&8': baselines 5-6 and 7-8
            antenna='5': all baselines with antenna 5
            antenna='5,6,10': all baselines including antennas 5, 6, or 10
            antenna='5,6,10&': all baselines with *only* antennas 5, 6, or
                                   10.  (cross-correlations only.  Use &&
                                   to include autocorrelations, and &&&
                                   to get only autocorrelations.)
            antenna='!ea03,ea12,ea17': all baselines except those that
                                       include EVLA antennas ea03, ea12, or
                                       ea17.

    correlation -- Correlation types or expression.
        default: '' (all correlations)
        example: correlation='XX,YY' 

    timerange -- Select data based on time range:
        default: '' (all); examples,
           timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
           Note: if YYYY/MM/DD is missing date, timerange defaults to the
           first day in the dataset
           timerange='09:14:0~09:54:0' picks 40 min on first day
           timerange='25:00:00~27:30:00' picks 1 hr to 3 hr 30min
           on next day
           timerange='09:44:00' data within one integration of time
           timerange='>10:24:00' data after this time

    array -- (Sub)array number range
        default: ''=all

    uvrange -- Select data within uvrange (default units meters)
        default: ''=all; example:
            uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lambda
            uvrange='>4klambda';uvranges greater than 4 kilo-lambda
            uvrange='0~1000km'; uvrange in kilometers

    observation -- Select by observation ID(s)
        default: ''=all

    feed -- Selection based on the feed - NOT IMPLEMENTED YET
        default: ''=all


    datacolumn -- Which data column to use for processing (case-insensitive).
        default: 'corrected'; example: datacolumn='data'
        options: 'data', 'model', 'corrected', 'all','float_data', 'lag_data', 
                 'float_data,data', 'lag_data,data'.

            N.B.: 'all' = whichever of the above that are present.
            Otherwise the selected column will go to DATA in the output.

            When datacolumn is set to either one of the values 'model','all',
            'data,model,corrected', a sub-parameter realmodelcol will be enabled.
            See description below.

        realmodelcol -- Make real a virtual MODEL column. If set to True, a real MODEL_DATA 
                        column will be added to the output MS based on the existing SOURCE_MODEL
                        column.
            default: False

    usewtspectrum -- Create a WEIGHT_SPECTRUM column in the output MS. When set to True,
                     a WEIGHT_SPECTRUM column will be created using the input WEIGHT column,
                     such that each channel in the WEIGHT_SPECTRUM will get WEIGHT/nChannels.
        default: False

--- SPW combination parameters ---
    combinespws -- Combine the input spws into a new output spw.
        default: False

            NOTE: Whenever the data to be combined has different EXPOSURE values 
                  in the spectral windows, mstransform will use the WEIGHT_SPECTRUM 
                  for the combination. If WEIGHT_SPECTRUM is not available, it will 
                  use the values from the WEIGHT column. Each output channel is calculated 
                  using the following equation:

          outputChannel_j = SUM(inputChannel_i*contributionFraction_i*inputWeightSpectrum_i) 
                           ------------------------------------------------------------------
                                 SUM(contributionFraction_i*inputWeightSpectrum_i)

--- Channel averaging parameters ---
    chanaverage -- Average data in channels. 
        default: False

        chanbin -- Number of input channels to average to create an output
                   channel. If a list is given, each bin will apply to one spw in
                   the selection.
            default: 1 => no channel averaging.
            options: (int) or [int]
            example: chanbin=[2,3] => average 2 channels of 1st selected
		             spectral window and 3 in the second one.

        useweights -- Type of weights to use in average. It will apply to all spws.
            default: 'flags'
            options: 'flags', 'spectrum'. When set to 'spectrum', it will use the
                               WEIGHT_SPECTRUM column for the weights.


--- Hanning smoothing parameters --- 
    hanning -- Hanning smooth frequency channel data to remove Gibbs ringing.
        default: False

--- Regrid parameters ---
    regridms -- Regrid the MS to a new spw, channel structure or frame.
        default: False

        mode -- Regridding mode.
            default: 'channel'; produces equidistant grid based on first selected channel.
            options: 'velocity', 'frequency', 'channel_b'.

            When set to velocity or frequency, it means that the channels must be specified
            in the respective units. When set to channel_b it means an alternative 'channel' 
            mode that does not force an equidistant grid. It is faster.

        nchan -- Number of channels in the output spw (int).
            default: -1

        start -- First channel to use in the output spw (depends on the mode)
            default: 0 --> when mode='channel'

            When mode='channel', 'start' means the first channel in the input spw
            to use when creating the output spw. When mode='frequency',
            'start' means the lowest frequency of the output spw. If this information
            is not available, leave it blank and mstransform will calculate it.

        width -- Width of input channels that are used to create an output channel. 
            default: 1

            Note that mstransform will only shift spws with channel widths of the same 
            sign in a single operation. If you are regridding spws with mixed positive 
            and negative channel widths, you should run this task separated for each 
            group of spws. You can verify the channel widths for your MS using 
            listobs for example, and looking at the SPW table, column ChanWid.
            
        nspw -- Number of output spws to create in the output MS/MMS (int). 
            default: 1  --> it means, do not separate the spws.

            One can regrid the MS or not and further separate the
            output into a given number of spws. Internally, the framework
            will combine the selected spws before separating them so that
            channel gaps and overlaps are taken into account. This parameter
            will create a regular grid of spws in the output MS. If nchan
            is set, it will refer to the number of output channels in each
            of the separated spws.

        interpolation -- Spectral interpolation method.
            default: 'linear'
            options: 'nearest', 'cubic', 'spline', 'fftshift'

        phasecenter -- Direction measure or FIELD_ID for the mosaic center.
            default: '' (first selected field)
            options: FIELD_ID (int) or center coordinate (str).
            NOTE: As int, it gives the FIELD ID for the mosaic center. If a string,
                  it gives the center coordinate, e.g. 'J2000 12h56m43.88s +21d41m00.1s'.                   

        restfreq -- Specify rest frequency to use for output.
            default: ''; occasionally it is necessary to set this.
            example1 for some VLA spectral line data.
            example2 for NH_3 (1,1) put restfreq='23.694496GHz'.

        outframe -- Output reference frame (case-insensitive).
            default: ''; it will keep the input reference frame.
            options: 'LSRK', 'LSRD', 'BARY', 'GALACTO', 'LGROUP', 'CMB', 'GEO', 'TOPO'.

        veltype -- Definition of velocity (as used in mode).
            default: 'radio'

--- Time averaging parameters ---
    timeaverage -- Average data in time.
        default: False

        timebin -- Bin width for time averaging.
            default: '1s'

        timespan -- Let the timebin span across scan, state or both.
                    State is equivalent to sub-scans. One scan may have several
                    state ids. For ALMA MSs, the sub-scans are limited to about
                    30s duration each. In these cases, the task will automatically
                    add state to the timespan parameter. To see the number of states
                    in an MS, use the msmd tool. See help msmd.

            default: '' (separate time bins by both of the above)
            options: 'scan', 'state', 'state,scan'

            examples: 
            timespan = 'scan'; can be useful when the scan number
                       goes up with each integration as in many WSRT MSs.
            timespan = ['scan', 'state']: disregard scan and state
                       numbers when time averaging.
            timespan = 'state,scan'; same as above.

        maxuvwdistance -- Provide a maximum separation of start-to-end baselines 
                          that can be included in an average. (int)
            default: 0.0 (given in meters)


------ EXAMPLES ------
	
More documentation on mstransform can be found here:
http://www.eso.org/~scastro/ALMA/casa/MST/MSTransformDocs/MSTransformDocs.html

1) Split out a single channel.
    mstransform(vis='ctb80-vsm.ms', outputvis='mychn.ms', datacolumn='data', spw='0:25')

2) Only combine the selected spws into a single output spw.
    mstransform(vis='Four_ants.ms', outputvis='myspw.ms', combinespws=True, spw='0~3')

3) Combine two spws and regrid one field, using two input channels to make one output.
    mstransform(vis='jupiter6cm.demo.ms',outputvis='test1.ms',datacolumn='DATA',field='11',
                spw='0,1', combinespws=True, regridms=True, nchan=1, width=2)

4) Combine 24 spws and regrid in frequency mode to create 21 output channels. Change the 
   phase center.
    mstransform(vis='g19_d2usb_targets_line.ms', outputvis='test2.ms', datacolumn='DATA',
                combinespws=True, regridms=True, mode='frequency', nchan=21, start='229587.0MHz',
                width='1600kHz', phasecenter="J2000 18h25m56.09 -12d04m28.20")

5) Only apply Hanning smoothing to MS.
    mstransform(vis='g19_d2usb_targets_line.ms', outputvis='test3.ms', datacolumn='DATA',
                hanning=True)
 
6) Change the reference frame and apply Hanning smoothing after combining all spws.
    mstransform(vis='g19_d2usb_targets_line.ms', outputvis='test4.ms', datacolumn='DATA',
                combinespws=True, regridms=True, mode="channel", outframe="BARY",
                phasecenter="J2000 18h25m56.09 -12d04m28.20", hanning = True)

7) Apply time averaging using a bin of 30 seconds on the default CORRECTED column.
    mstransform(vis='g19_d2usb_targets_line.ms', outputvis='test5.ms', timeaverage=True,
                timebin='30s')



        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'mstransform'
        myf['taskname'] = 'mstransform'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['outputvis'] = outputvis = myf['outputvis']
            myparams['createmms'] = createmms = myf['createmms']
            myparams['separationaxis'] = separationaxis = myf['separationaxis']
            myparams['numsubms'] = numsubms = myf['numsubms']
            myparams['parallel'] = parallel = myf['parallel']
            myparams['ddistart'] = ddistart = myf['ddistart']
            myparams['tileshape'] = tileshape = myf['tileshape']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['scan'] = scan = myf['scan']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['correlation'] = correlation = myf['correlation']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['intent'] = intent = myf['intent']
            myparams['array'] = array = myf['array']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['observation'] = observation = myf['observation']
            myparams['feed'] = feed = myf['feed']
            myparams['datacolumn'] = datacolumn = myf['datacolumn']
            myparams['realmodelcol'] = realmodelcol = myf['realmodelcol']
            myparams['usewtspectrum'] = usewtspectrum = myf['usewtspectrum']
            myparams['combinespws'] = combinespws = myf['combinespws']
            myparams['chanaverage'] = chanaverage = myf['chanaverage']
            myparams['chanbin'] = chanbin = myf['chanbin']
            myparams['useweights'] = useweights = myf['useweights']
            myparams['hanning'] = hanning = myf['hanning']
            myparams['regridms'] = regridms = myf['regridms']
            myparams['mode'] = mode = myf['mode']
            myparams['nchan'] = nchan = myf['nchan']
            myparams['start'] = start = myf['start']
            myparams['width'] = width = myf['width']
            myparams['nspw'] = nspw = myf['nspw']
            myparams['interpolation'] = interpolation = myf['interpolation']
            myparams['phasecenter'] = phasecenter = myf['phasecenter']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['outframe'] = outframe = myf['outframe']
            myparams['veltype'] = veltype = myf['veltype']
            myparams['timeaverage'] = timeaverage = myf['timeaverage']
            myparams['timebin'] = timebin = myf['timebin']
            myparams['timespan'] = timespan = myf['timespan']
            myparams['maxuvwdistance'] = maxuvwdistance = myf['maxuvwdistance']

        if type(tileshape)==int: tileshape=[tileshape]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['outputvis'] = outputvis
        mytmp['createmms'] = createmms
        mytmp['separationaxis'] = separationaxis
        mytmp['numsubms'] = numsubms
        mytmp['parallel'] = parallel
        mytmp['ddistart'] = ddistart
        mytmp['tileshape'] = tileshape
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['scan'] = scan
        mytmp['antenna'] = antenna
        mytmp['correlation'] = correlation
        mytmp['timerange'] = timerange
        mytmp['intent'] = intent
        mytmp['array'] = array
        mytmp['uvrange'] = uvrange
        mytmp['observation'] = observation
        mytmp['feed'] = feed
        mytmp['datacolumn'] = datacolumn
        mytmp['realmodelcol'] = realmodelcol
        mytmp['usewtspectrum'] = usewtspectrum
        mytmp['combinespws'] = combinespws
        mytmp['chanaverage'] = chanaverage
        mytmp['chanbin'] = chanbin
        mytmp['useweights'] = useweights
        mytmp['hanning'] = hanning
        mytmp['regridms'] = regridms
        mytmp['mode'] = mode
        mytmp['nchan'] = nchan
        mytmp['start'] = start
        mytmp['width'] = width
        mytmp['nspw'] = nspw
        mytmp['interpolation'] = interpolation
        mytmp['phasecenter'] = phasecenter
        mytmp['restfreq'] = restfreq
        mytmp['outframe'] = outframe
        mytmp['veltype'] = veltype
        mytmp['timeaverage'] = timeaverage
        mytmp['timebin'] = timebin
        mytmp['timespan'] = timespan
        mytmp['maxuvwdistance'] = maxuvwdistance
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'mstransform.xml')

        casalog.origin('mstransform')
        if not trec.has_key('mstransform') or not casac.utils().verify(mytmp, trec['mstransform']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: mstransform           #####')
          casalog.post('')
          result = mstransform(vis, outputvis, createmms, separationaxis, numsubms, parallel, ddistart, tileshape, field, spw, scan, antenna, correlation, timerange, intent, array, uvrange, observation, feed, datacolumn, realmodelcol, usewtspectrum, combinespws, chanaverage, chanbin, useweights, hanning, regridms, mode, nchan, start, width, nspw, interpolation, phasecenter, restfreq, outframe, veltype, timeaverage, timebin, timespan, maxuvwdistance)
          casalog.post('')
          casalog.post('##### End Task: mstransform           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('mstransform', 'mstransform.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'mstransform'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('mstransform', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['outputvis']  = ''
        a['createmms']  = False
        a['tileshape']  = [0]
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['antenna']  = ''
        a['correlation']  = ''
        a['timerange']  = ''
        a['intent']  = ''
        a['array']  = ''
        a['uvrange']  = ''
        a['observation']  = ''
        a['feed']  = ''
        a['datacolumn']  = 'corrected'
        a['usewtspectrum']  = False
        a['combinespws']  = False
        a['chanaverage']  = False
        a['hanning']  = False
        a['regridms']  = False
        a['timeaverage']  = False

        a['async']=False
        a['createmms'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'separationaxis':'both'}, {'numsubms':64}, {'parallel':False}])}
        a['datacolumn'] = {
                    0:{'value':'corrected'}, 
                    1:odict([{'value':'model'}, {'realmodelcol':False}]), 
                    2:odict([{'value':'all'}, {'realmodelcol':False}]), 
                    3:odict([{'value':'data,model,corrected'}, {'realmodelcol':False}])}
        a['chanaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'chanbin':1}, {'useweights':'flags'}])}
        a['regridms'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'mode':'channel'}, {'nchan':-1}, {'start':0}, {'width':1}, {'nspw':1}, {'interpolation':'linear'}, {'phasecenter':''}, {'restfreq':''}, {'outframe':''}, {'veltype':'radio'}])}
        a['timeaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'timebin':'0s'}, {'timespan':''}, {'maxuvwdistance':0.0}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='mstransform', subkey=None):
        desc={'mstransform': 'Split the MS, combine/separate/regrid spws and do channel and time averaging',
               'vis': 'Name of input Measurement set or Multi-MS.',
               'outputvis': 'Name of output Measurement set or Multi-MS.',
               'createmms': 'Create a multi-MS output.',
               'separationaxis': 'Axis to do parallelization across(scan,spw,both).',
               'numsubms': 'The number of Sub-MSs to create.',
               'parallel': 'Create a multi-MS in parallel using simple_cluster.',
               'ddistart': 'For internal use only. Do not change it!',
               'tileshape': 'List with 1 or 3 elements giving the tile shape of the disk data columns.',
               'field': 'Select field using ID(s) or name(s).',
               'spw': 'Select spectral window/channels.',
               'scan': 'Select data by scan numbers.',
               'antenna': 'Select data based on antenna/baseline.',
               'correlation': 'Correlation: \'\' ==> all, correlation=\'XX,YY\'.',
               'timerange': 'Select data by time range.',
               'intent': 'Select data by scan intent.',
               'array': 'Select (sub)array(s) by array ID number.',
               'uvrange': 'Select data by baseline length.',
               'observation': 'Select by observation ID(s).',
               'feed': 'Multi-feed numbers: Not yet implemented.',
               'datacolumn': 'Which data column(s) to process.',
               'realmodelcol': 'Make real a virtual MODEL column.',
               'usewtspectrum': 'Create a WEIGHT_SPECTRUM column in the output MS.',
               'combinespws': 'Combine the input spws into a new output spw.',
               'chanaverage': 'Average data in channels.',
               'chanbin': 'Width (bin) of input channels to average to form an output channel.',
               'useweights': 'Type of weights to use in average (flags/spectrum).',
               'hanning': 'Hanning smooth data to remove Gibbs ringing.',
               'regridms': 'Regrid the MS to a new spw, channel structure or frame.',
               'mode': 'Regridding mode (channel/velocity/frequency/channel_b).',
               'nchan': 'Number of channels in the output spw (-1=all).',
               'start': 'First channel to use in the output spw (mode-dependant)',
               'width': 'Number of input channels that are used to create an output channel.',
               'nspw': 'Number of output spws to create in output MS.',
               'interpolation': 'Spectral interpolation method.',
               'phasecenter': 'Image phase center: position or field index.',
               'restfreq': 'Rest frequency to use for output.',
               'outframe': 'Output reference frame (\'\'=keep input frame).',
               'veltype': 'Velocity definition.',
               'timeaverage': 'Average data in time.',
               'timebin': 'Bin width for time averaging.',
               'timespan': 'Span the timebin across scan, state or both.',
               'maxuvwdistance': 'Maximum separation of start-to-end baselines that can be included in an average. (meters)',

               'async': 'If true the taskname must be started using mstransform(...)'
              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['outputvis']  = ''
        a['createmms']  = False
        a['separationaxis']  = 'both'
        a['numsubms']  = 64
        a['parallel']  = False
        a['ddistart']  = -1
        a['tileshape']  = [0]
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['antenna']  = ''
        a['correlation']  = ''
        a['timerange']  = ''
        a['intent']  = ''
        a['array']  = ''
        a['uvrange']  = ''
        a['observation']  = ''
        a['feed']  = ''
        a['datacolumn']  = 'corrected'
        a['realmodelcol']  = False
        a['usewtspectrum']  = False
        a['combinespws']  = False
        a['chanaverage']  = False
        a['chanbin']  = 1
        a['useweights']  = 'flags'
        a['hanning']  = False
        a['regridms']  = False
        a['mode']  = 'channel'
        a['nchan']  = -1
        a['start']  = 0
        a['width']  = 1
        a['nspw']  = 1
        a['interpolation']  = 'linear'
        a['phasecenter']  = ''
        a['restfreq']  = ''
        a['outframe']  = ''
        a['veltype']  = 'radio'
        a['timeaverage']  = False
        a['timebin']  = '0s'
        a['timespan']  = ''
        a['maxuvwdistance']  = 0.0

        if a.has_key(paramname) :
	      return a[paramname]
mstransform_pg = mstransform_pg_()
